"use strict";
exports.__esModule = true;
var Venti = /** @class */ (function () {
    function Venti() {
        this.eventLogLimit = 1000;
        this.registeredEvents = {};
        this.log = [];
    }
    Venti.instance = function () {
        if (!Venti._instance) {
            Venti._instance = new Venti();
        }
        return Venti._instance;
    };
    Venti.prototype.trigger = function (str, data) {
        var _this = this;
        if (data === void 0) { data = undefined; }
        var events = this.registeredEvents[str];
        if (events) {
            events.forEach(function (e) {
                e(data);
                _this.log.push({ event: str, callback: _this.nameFromFunction(e) });
            });
        }
        else {
            console.warn('Venti - Notice: You tried to emit an event (' + str + ') which is not registered.');
        }
    };
    Venti.prototype.nameFromFunction = function (callback) {
        var ret = callback.toString();
        ret = ret.substring('function '.length);
        return ret.substring(0, ret.indexOf('('));
    };
    Venti.prototype.eventLog = function (limit) {
        var l = this.eventLogLimit;
        if (typeof limit !== 'undefined') {
            l = limit;
        }
        return this.log.reverse().splice(0, l);
    };
    Venti.prototype.on = function (str, callback) {
        if (typeof this.registeredEvents[str] === 'undefined') {
            this.registeredEvents[str] = [];
        }
        if (this.registeredEvents[str].indexOf(callback) !== -1) {
            if (typeof callback == 'undefined') {
                console.error('Venti - You have declared an undefined function inside your .on function for event ' + str);
            }
            else {
                console.warn('Venti - Notice: You tried to re-link a function (' + this.nameFromFunction(callback) + ') that was already registered to this event.');
            }
            delete this.registeredEvents[str][this.registeredEvents[str].indexOf(callback)];
        }
        this.registeredEvents[str].push(callback);
        this.registeredEvents[str] = this.registeredEvents[str].filter(function () { return true; });
    };
    Venti.prototype.off = function (str, callback) {
        if (callback) {
            if (typeof this.registeredEvents[str] !== 'undefined') {
                delete this.registeredEvents[str][this.registeredEvents[str].indexOf(callback)];
            }
        }
        else {
            if (typeof this.registeredEvents[str] !== 'undefined') {
                delete this.registeredEvents[str];
            }
        }
    };
    return Venti;
}());
exports["default"] = Venti.instance();
